#!/bin/bash

. /usr/local/Reductor/etc/const
. $CONFIG

set -eu

trap __exit EXIT

# теперь можем спокойно использовать этот враппер везде, скорость load НЕ ВАЖНА из-за double buffer
write2proc() {
	echo "$@" > $PROCLIST
}

load_url_and_signatures_to_proc() {
	local failed=0
	echo >> $LOGFILE || true
	log "INFO: начинаем загрузку URL в ядро" >> $LOGFILE || true
	while read sign url; do
		if ! write2proc "$sign $1 $url"; then
			log "ERROR: не удалось добавить _ $sign $1 $url _"
			((failed++)) || true
		fi
	done < "$2"
	log "Загрузка URL в ядро закончена" >> $LOGFILE || true
	[ "$failed" -gt '0' ] && log "ERROR: $failed URL не было добавлено" | tee -a $LOGFILE || true
	echo >> $LOGFILE || true
	return 0
}

load_signatures_and_cache() {
	write2proc clear
	for list in $(url_lists); do
		echo "  - $list"
		# -t <TAB>, not a few spaces
		if ! LANG='' join -t '	' -1 1 -2 2 -o "2.1 1.1" $list $CACHEDIR/signatures.cache > $TMPDIR/signatures_and_urls.tmp; then
			log "ERROR: Ошибка при обработке списка $list" | tee -a $LOGFILE || true
			continue
		fi
		grep -v ^clear $TMPDIR/signatures_and_urls.tmp > $TMPDIR/signatures_and_urls || true
		list_id=${list##*/}
		[ "$list" = "$LISTDIR/http.load" ] && list_id=0 || true
		load_url_and_signatures_to_proc $list_id $TMPDIR/signatures_and_urls
	done
	write2proc switch
}

use_hook() {
	hook="$HOOKDIR/${0##*/}"
	if [ -x "$hook" ]; then
		. "$hook"
	fi
}

client_prepare_hook() {
	return 0
}

prepare_lists_if_need() {
	if [ ! -s $LISTDIR/rkn.list -a ! -s $LISTDIR/rkn.iplist -a -s $DUMPXML ]; then
		echo "- Разбираем единый реестр запрещённых сайтов"
		$BINDIR/dump_parser.sh
	fi

	if [ ! -s $LISTDIR/http.load ]; then
		echo "- Агрегируем списки"
		$BINDIR/prepare_lists.sh
	fi
}

load_dns() {
	local list
	if ! [ -s $LISTDIR/https.resolv ]; then
		echo "- WARNING: Пустой $LISTDIR/https.resolv"
		return 0
	fi
	$BINDIR/dnsenabled || return 0
	local PROC_DNS="/proc/net/ipt_dnsmatch/block_list"
	echo "- Загрузка доменов в ядро"
	echo "clear" > "$PROC_DNS"
	for list in $(domain_lists); do
		echo "  - $list"
		list_id=0
		exact="${filter['dns_exact']:-0}"
		if [ "$list" != "$LISTDIR/https.resolv" ]; then
			list_id="${list##*/}"
			read _ _ _ exact _ <<< "$(grep "^$list_id " userinfo/hooks/custom_bindings_dns)" || continue
		fi
		python -u $BINDIR/dnsload.py "$list" "$list_id" "$exact" | while read domain; do
			echo -n "$domain" > "$PROC_DNS"
		done
	done
	echo "switch" > "$PROC_DNS"
}

main() {
	prepare_lists_if_need
	client_prepare_hook
	echo "- Загрузка ipset"
	$BINDIR/load_ip.sh
	echo "- Загрузка URL в ядро"
	load_signatures_and_cache
	load_dns
}

use_hook
main "$@"
